using System;

	/// <summary>
	/// The Spanish Stemmer class stems words in spanish based on the Snowball algorithm
	/// but modified to add some functionality.
	/// </summary>
	public class SpanishStemmer
	{
		public SpanishStemmer()
		{
		}

		private static bool isVowel(char eval)
		{
			bool res = false;
			char[] vowels = new char[] {'a','e','i','o','u','','','','','',''};

			foreach( char c in vowels )
			{ if (eval == c) {res = true; break;} }

			return res;
		}

		public static string stemWord(string origword)
		{
			string completeword = origword.ToLower();
			string RV = "";

			// Check if attached pronouns could be found.
			if (origword.Length > 3) 
			{	// I'll include the last letter before, so we can test precedence without checking the original.
				RV = origword.Substring(2, origword.Length - 2);
			}
			else {RV = "";}

			// Eliminate attached pronouns
			// In this order:  (added to Snowball Algorithm :  te, telo, melo, telos, melos, tela, mela, telas, melas ).
			string[] pronouns = new string[] {	"me","se", "te", "telo", "melo", "telos", "melos", "tela", "mela", "telas", 
												 "melas", "sela","selo","selas","selos","la","le","lo","las","les","los","nos"};
			// if it comes after any of:
			string[] prevcheck = new string[] {"indo","ndo","r","r","r","iendo","ando","ar","er","ir","yendo"};
			char[] preceding = new char[] {'0','0','0','0','0','0','0','0','0','0','u'};
			string[] replacement = new string[] {"iendo","ando","ar","er","ir","","","","","",""};

			foreach(string s in pronouns)
			{
				int marker = RV.Length - s.Length;
				if (marker >= 0 && RV.Substring(marker, s.Length) == s)
				{	
					bool mustRemove = false;
					string mustReplace = "";
					for (int i = 0; i < prevcheck.Length; i++)
					{
						int pos = marker - prevcheck[i].Length;
						if (pos > 0)
						{
							string tmp = RV.Substring(pos, prevcheck[i].Length);
							if (( tmp == prevcheck[i]) && ( (preceding[i] == '0') || ( ( pos > 0 ) && (preceding[i] == prevcheck[i][pos-1] ))))
							{
								mustRemove = true; 
								mustReplace = replacement[i];
								break;}
						}
					}
					if (mustRemove) 
					{
						completeword = completeword.Remove(marker + 2, s.Length);
						RV = RV.Remove(marker, s.Length);
						if (mustReplace != "") 
						{
							completeword = completeword.Remove( completeword.Length - mustReplace.Length, mustReplace.Length ); 
							completeword += mustReplace;
							RV = RV.Remove( RV.Length - mustReplace.Length, mustReplace.Length);
							RV += mustReplace;
						}
						break;  }
				}
			}
			
			string copy = completeword;

			// Then, remove standard suffixes
			string[] suffixes = new string[] {"anza","anzas","ico","ica","icos","icas","ismo","ismos","able","ables",
												 "ible","ibles","ista","istas","oso","osa","osos","osas","amiento","amientos","imiento","imientos",
												 "icadora","icador","icacin","icadoras","icadores","icaciones","icante","icantes","icancia","icancias",
												 "adora","ador","acin","adoras","adores","aciones","ante","antes","ancia","ancias",
												 "loga","logas",
												 "ucin","uciones",
												 "encia","encias",
												 "ativamente","ivamente","osamente","icamente","adamente","amente",
												 "antemente","ablemente","iblemente","mente",
												 "abilidad","abilidades","icidad","icidades","ividad","ividades","idad","idades",
												 "ativa","ativo","ativas","ativos","iva","ivo","ivas","ivos"};

			string[] replacesuf = new string[] {"","","","","","","","","","","","","","","","","","","","","","",
												   "","","","","","","","","","",
												   "","","","","","","","","","",
												   "log","log",
												   "u","u",
												   "ente","ente",
												   "","","","","","",
												   "","","","",
												   "","","","","","","","",
												   "","","","","","","",""};
			
			for(int i = 0; i < suffixes.Length; i++ )
			{
				int marker = RV.Length - suffixes[i].Length;
				if (marker >= 0 && RV.Substring(marker, suffixes[i].Length) == suffixes[i])
				{
					completeword = completeword.Remove(marker + 2, suffixes[i].Length);
					RV = RV.Remove(marker, suffixes[i].Length);

					string mustReplace = replacesuf[i];
					if (mustReplace != "") 
					{
						completeword += mustReplace;
						RV += mustReplace;
					}
					break;  
				}						
			}

			// If no change was made...
			if (copy == completeword)
			{
				string[] suffixes2 = new string[] {"ya","ye","yan","yen","yeron","yendo","yo","y","yas","yes","yais","yamos"};
				char precedence = 'u';
				bool failtoremove = true;

				foreach(string s in suffixes2)
				{
						int marker = RV.Length - s.Length;
					if ( marker >=0 && RV.Substring(marker, s.Length) == s)
					{
                        if ((RV.Length > s.Length) && (RV[RV.Length - s.Length - 1] == precedence))
						{
							completeword = completeword.Remove(completeword.Length - s.Length, s.Length);
							RV = RV.Remove(RV.Length - s.Length, s.Length);
							failtoremove = false; }
					}
				}
				if (failtoremove)
				{
					string[] othersuf = new string[] {"en","es","is","emos","guen","gues","guis","guemos",
														 "aran","aras","arn","ars","arais","ara","aris","aramos","aremos","ar","ar",
														 "eran","eras","ern","ers","erais","era","eris","eramos","eremos","er","er",
														 "iran","iras","irn","irs","irais","ira","iris","iramos","iremos","ir","ir",
														 "aba","ada","ida","a","ara","iera","ad","ed","id","ase","iese","aste",
														 "iste","an","aban","an","aran","ieran","asen","iesen","aron","ieron","ado","ido",
														 "ando","iendo","i","ar","er","ir","as","abas","adas","idas","as","aras","ieras","ases",
														 "ieses","s","is","abais","ais","arais","ierais","aseis","ieseis","asteis","isteis",
														 "ados","idos","amos","bamos","amos","imos","ramos","iramos","isemos","semos"};
					string[] replace = new string[] {"","","","","g","g","g","g",
														"","","","","","","","","","","","","","","","",
														"","","","","","","","","","","","","","","","",
														"","","","","","","","","","","","","","","","",
														"","","","","","","","","","","","","","","","",
														"","","","","","","","","","","","","","","","",
														"","","","","","","","","","","",""};
					
					for (int i = 0; i < othersuf.Length; i++)
					{
						int marker = RV.Length - othersuf[i].Length;
						if(marker >= 0 && RV.Substring(marker , othersuf[i].Length) == othersuf[i])
						{
							completeword = completeword.Remove(completeword.Length - othersuf[i].Length, othersuf[i].Length);
							RV = RV.Remove(RV.Length - othersuf[i].Length, othersuf[i].Length);		
							if (replace[i] != "") 
							{
								completeword += replace[i];
								RV += replace[i];	}
							break;
						}
					}
				}
			}
		
			// Diminutives/Superlatives (Step 4) & Residual Suffix (Step 3)
			string[] ressuf = new string[] {"ito", "ita", "azo", "aza", "lin", "lina", "in",  // Step 4
											"ina", "on", "ona", "itillo", "itilla", "cillo",
											"cilla", "illo", "illa", "itico", "itica", "ico",
											"ica", "ote", "ota",
											// Step 3
											"os","a","o","","","","e","","gue","gu"
											};	
			string[] repsuf = new string[] { 
											"", "", "", "", "", "", "",
											"", "", "", "", "", "",
											"", "", "", "", "", "",
											"", "", "",
											"","","","","","","","","g","g"};	
			
			for (int i = 0; i < ressuf.Length; i++)
			{
				int marker = RV.Length - ressuf[i].Length;
				if(marker >= 0 && RV.Substring(marker, ressuf[i].Length) == ressuf[i])
				{
					completeword = completeword.Remove(completeword.Length - ressuf[i].Length, ressuf[i].Length);
					RV = RV.Remove(RV.Length - ressuf[i].Length, ressuf[i].Length);		
					if (repsuf[i] != "") 
					{
						completeword += repsuf[i];
						RV += repsuf[i];	}
					break;
				}
			}

			// Finally, remove the accents.
			completeword = completeword.Replace("","a");
			completeword = completeword.Replace("","e");
			completeword = completeword.Replace("","i");
			completeword = completeword.Replace("","o");
			completeword = completeword.Replace("","u");
			
			return completeword;
		}
	}

